/*!
 * Copyright (c) Microsoft Corporation and contributors. All rights reserved.
 * Licensed under the MIT License.
 */
import { ISegment, Marker } from "./mergeTree";
import { ICombiningOp, IMergeTreeAnnotateMsg, IMergeTreeGroupMsg, IMergeTreeInsertMsg, IMergeTreeRemoveMsg, IMergeTreeDeltaOp } from "./ops";
import { PropertySet } from "./properties";
/**
 * Creates the op for annotating the markers with the provided properties
 * @param marker - The marker to annotate
 * @param props - The properties to annotate the marker with
 * @param combiningOp - Optional. Specifies how to combine values for the property, such as "incr" for increment.
 * @returns The annotate op
 */
export declare function createAnnotateMarkerOp(marker: Marker, props: PropertySet, combiningOp: ICombiningOp): IMergeTreeAnnotateMsg | undefined;
/**
 * Creates the op for annotating the range with the provided properties
 * @param start - The inclusive start postition of the range to annotate
 * @param end - The exclusive end position of the range to annotate
 * @param props - The properties to annotate the range with
 * @param combiningOp - Optional. Specifies how to combine values for the property, such as "incr" for increment.
 * @returns The annotate op
 */
export declare function createAnnotateRangeOp(start: number, end: number, props: PropertySet, combiningOp: ICombiningOp | undefined): IMergeTreeAnnotateMsg;
/**
 * Creates the op to remove a range and puts the content of the removed range in a register
 * if a register name is provided
 *
 * @param start - The inclusive start of the range to remove
 * @param end - The exclusive end of the range to remove
 * @param register - Optional. The name of the register to store the removed range in
 */
export declare function createRemoveRangeOp(start: number, end: number, register?: string): IMergeTreeRemoveMsg;
/**
 *
 * @param pos - The position to insert the segment at
 * @param segment - The segment to insert
 */
export declare function createInsertSegmentOp(pos: number, segment: ISegment): IMergeTreeInsertMsg;
export declare function createInsertOp(pos: number, segSpec: any): IMergeTreeInsertMsg;
/**
 *
 * @param pos - The position to insert the register contents at
 * @param register - The name of the register to insert the value of
 */
export declare function createInsertFromRegisterOp(pos: number, register: string): IMergeTreeInsertMsg;
/**
 *
 * @param start - The inclusive start of the range to insert into the register
 * @param end - The exclusive end of the range to insert into the register
 * @param register - The name of the register to insert the range contents into
 */
export declare function createInsertToRegisterOp(start: number, end: number, register: string): IMergeTreeInsertMsg;
/**
 *
 * @param ops - The ops to group
 */
export declare function createGroupOp(...ops: IMergeTreeDeltaOp[]): IMergeTreeGroupMsg;
//# sourceMappingURL=opBuilder.d.ts.map